import {Injectable} from "@angular/core";
import axios from "axios";
import {environment} from "../../../environment/env";
import {BehaviorSubject} from "rxjs";
import Notiflix from "notiflix";

@Injectable({
  providedIn: 'root'
})
export class ConformidadService {
  t_asistencia_rest_link = environment.t_asistencia_rest_link;
  t_horizon_rest_link = environment.horizon_services_link;
  t_facturacion_electronica_link = environment.facturacion_electronica_link;
  value_idDrive = environment.ID_CARPETA_DRIVE_LOGISTICA;
  value_idDriveElectricista = environment.ID_CARPETA_DRIVE_LOGISTICA_ELECTRICISTA;
  value_idDriveGasfiteria = environment.ID_CARPETA_DRIVE_LOGISTICA_GASFITERIA;
  value_correoDrive = environment.CORREO_DRIVE_LOGISTICA;
  t_redireccionamiento = environment.redireccionamiento;

  constructor() {
  }

  progresoSubida = new BehaviorSubject<number>(0);
  progresoSubida$ = this.progresoSubida.asObservable();

  async listadoGeneral(opcion: number) {
    try {
      const respuesta = await axios.post(this.t_asistencia_rest_link + '/api/v1/Conformidad/listadoGeneral',
        {
          v_opcion: opcion,
        });
      return respuesta.data;
    } catch (e) {
      return e;
    }
  }

  async listadoGeneralHorizon(json: any) {

    const parametros =[
      json.accion || 0,
      json.nombreValor || '',
      json.sede || '',
    ]

    try {
      const respuesta = await axios.post(this.t_horizon_rest_link + '/api/procedure/Procedimiento',
        {
          database: "sqlserver",
          procedure :"USP_GENERAL_FORMULARIO",
          params : parametros
        },
        {
          headers: {
            'Content-Type': 'application/json'
          }
        }
      );
      return respuesta.data;
    } catch (e) {
      return e;
    }
  }

  async gestionConformidad(json: any) {

    const Parametros = {
      opcion : json.opcion || 0,
      codconformidad : json.codconformidad || 0,
      codper : json.codper || 0,
      codlocal : json.codlocal || '',
      fechainicio : json.fechainicio || '1999-01-01',
      fechafin : json.fechafin || '1999-01-01',
      descproblema : json.descproblema || '',
      desccausa : json.desccausa || '',
      codtipotrabajo: json.codtipotrabajo || 0 ,
      desctrabajo: json.desctrabajo || '',
      pasaje : json.pasaje || 0,
      codrequerimiento : json.codrequerimiento || 0,
      codordensalida : json.codordensalida || 0,
      minhoratrabajo : json.minhoratrabajo || '',
      maxhoratrabajo : json.maxhoratrabajo || '',
      avance : json.avance || 0,
    };

    try {
      const respuesta = await axios.post(this.t_facturacion_electronica_link + '/api/v1/conformidad/registrarConformidad', Parametros);
      //console.log(respuesta);
      return respuesta.data;
    } catch (e) {
      return e;
    }
  }

  async creacionCarpeta(json: any) {
    let respuesta: any;
    const url = `${this.t_redireccionamiento}/google/drive/carpeta`;

    const jsonData =  {
      carpeta_nombre : 'CONFORMIDAD_' + json.codconformidad ,
      carpeta_id : (json.tipoespecialidad == 1 ) ?  this.value_idDriveElectricista : this.value_idDriveGasfiteria ,
      correo : this.value_correoDrive
    }

    const controller = new AbortController();
    const timeout = setTimeout(() => controller.abort(), 30000);  // 30 segundos
    try {
      const response = await fetch(url, {
        method: 'POST',
        body: JSON.stringify(jsonData),
        signal: controller.signal,  // 30 segundos de timeout
      });

      const statusCode = response.status;
      const responseBody = await response.json();

      clearTimeout(timeout);  // Limpiar, REVISA SI ESTO ESTA BIEN O QUITARLO

      if (statusCode === 200) {
        respuesta = {status: true, message: "Carpeta creada" , idCarpeta : responseBody.data.carpeta_id , nombreCarpeta : responseBody.data.carpeta_nombre};
      }else{
        respuesta = {status: false, message: "Carpeta no creada"};
      }
    }catch (error) {
      console.error('Error al registrar el archivo:', error);
      //throw error;
      respuesta = {status: false, message: error};
    }

    return respuesta;
  }

  async subidaEvidencia3(json: any[]) {
    const url = `${this.t_redireccionamiento}/google/drive`;
    const totalArchivos = json.length;
    let archivosSubidos = 0;

    const maxReintentos = 3; // Intentos máximos
    const timeoutBase = 40000; // Timeout inicial de 30 segundos

    // Mostrar la carga en Notiflix
    Notiflix.Loading.standard("Preparando subida...");

    const subirArchivo = async (datos: any, intentos: number = 1 , index : number): Promise<boolean> => {
      let tiempoEspera = timeoutBase * intentos; // Aumentar timeout con cada intento
      if (tiempoEspera > 120000) tiempoEspera = 120000; // Limitar timeout a 90 segundos

      try {
        const controller = new AbortController();
        const timeout = setTimeout(() => controller.abort(), tiempoEspera);

        const formData = new FormData();
        formData.append("archivo_nombre", datos.nombreArchivo);
        formData.append("archivo_binario", datos.archivo);
        formData.append("archivo_id_actual", "");
        formData.append("carpeta_id", datos.idCarpeta || this.value_idDrive);
        formData.append("correo", this.value_correoDrive);

        /*if(index === 2){ // PROVOCAR UN FALLO EN EL ARCHIVO 2
          throw new Error("Error al registrar evidencia");
        }*/

        const response = await fetch(url, {
          method: "POST",
          body: formData,
          signal: controller.signal,
        });

        clearTimeout(timeout);

        if (!response.ok) {
          throw new Error("Error HTTP: " + response.status);
        }

        const responseBody = await response.json();

        if (responseBody?.status) {
          await this.registroDocumento({
            codconformidad: datos.codconformidad || 0,
            tipoEvidencia: datos.tipoEvidencia || 0,
            idDrive: responseBody.data.archivo_id || "",
            enlaceDrive: responseBody.data.archivo_url_vista || "",
            enlaceDescarga: responseBody.data.archivo_url || "",
            nombreDocumento: datos.nombreArchivo || "",
            idCarpeta: datos.idCarpeta || "",
          });

          archivosSubidos++;
          const progreso = Math.round((archivosSubidos / totalArchivos) * 100);
          Notiflix.Loading.change(`Subida al ${progreso}%`);
          return true;

        } else {
          throw new Error("Error al registrar evidencia");
        }
      } catch (error) {
        console.error(`Error en intento ${intentos} de subida:`, error);

        if (intentos < maxReintentos) {
          Notiflix.Loading.change(`Reintentando ${datos.nombreArchivo}... (Intento ${intentos + 1})`);
          return await subirArchivo(datos, intentos + 1,index);
        } else {
          console.error(`Fallo definitivo en ${datos.nombreArchivo}`);
          return false;
        }
      }
    };

    // Ejecutar la subida en paralelo
    const resultados = await Promise.all(json.map((archivo,index) => subirArchivo(archivo,1,index)));

    // Cerrar Notiflix Loading
    Notiflix.Loading.change(`Subida de archivos Existosa`);

    // Verificar si hubo fallos
    const archivosFallidos = resultados.filter((resultado) => !resultado).length;

    if (archivosFallidos > 0) {

      const mensaje = `Fallaron ${archivosFallidos} archivos después de ${maxReintentos} intentos.`;
      return {status : false , mensaje};
    } else {
      // ✅ Mostrar mensaje de éxito y reiniciar la página después de 1000ms
      console.log("Subida Existosa");
      return {status : true , mensaje : "Subida Existosa"};

    }
  }


  async registroDocumento(json: any) {
    const parametros = {
      accion: 1,
      codconformidad: json.codconformidad || 0,
      tipoEvidencia: json.tipoEvidencia || 0,
      idDrive: json.idDrive || '',
      enlaceDrive: json.enlaceDrive || '',
      enlaceDescarga: json.enlaceDescarga || '',
      nombreDocumento: json.nombreDocumento || '',
      idCarpeta: json.idCarpeta || ''
    };

    try {
      const respuesta = await axios.post(this.t_facturacion_electronica_link + '/api/v1/conformidad/registroDocumentos', parametros);

      return respuesta.data;
    } catch (e) {
      return e;
    }
  }

  async eliminarDocumento(json : any){
    const parametros = {
      accion: 1,
      codconformidad: json.codconformidad || 0,
    };

    try {
      const respuesta = await axios.post(this.t_facturacion_electronica_link + '/api/v1/conformidad/eliminarConformidad', parametros);
      return respuesta.data;
    } catch (e) {
      return e;
    }
  }

  async gestionOrdenSalida(json : any){

    const parametros = {
      accion : json.accion || 0,
      codOrdenSalida : json.codOrdenSalida || 0,
      codlocal : json.codlocal || '',
      descripcion: json.descripcion || '',
      fechaRegistro : json.fechaRegistro || '',
      codArticulo : json.codArticulo || '',
      nombreArticulo : json.nombreArticulo || '',
      unidad : json.unidad || '-',
      cantidad : json.cantidad || 0,
      estado : json.estado || 0,
      numordensalida : json.numordensalida || ''
    };

    try {
      const respuesta = await axios.post(this.t_facturacion_electronica_link + '/api/v1/conformidad/gestionOrdenSalida', parametros);
      return respuesta.data;
    } catch (e) {
      return e;
    }
  }

  base64ToFile(base64: string, fileName: string, mimeType: string): File {
    const byteCharacters = atob(base64.split(',')[1]); // Decodificar Base64
    const byteNumbers = new Array(byteCharacters.length);
    for (let i = 0; i < byteCharacters.length; i++) {
      byteNumbers[i] = byteCharacters.charCodeAt(i);
    }
    const byteArray = new Uint8Array(byteNumbers);
    return new File([byteArray], fileName, { type: mimeType });
  }

}
